import asyncio
import os
import posixpath
import shutil
from pathlib import Path
from typing import List, Optional

import click

from honesty.cache import Cache
from honesty.releases import FileType, Package, parse_index


def download_many(
    package: Package, versions: List[str], dest: Path, cache: Cache
) -> int:
    """
    Intended as a convenience method for the CLI.  If you want async duplicate
    this.  Version parsing happens in the layer above in cmdline.py.
    """
    loop = asyncio.get_event_loop()
    return loop.run_until_complete(async_download_many(package, versions, dest, cache))


async def async_download_many(
    package: Package, versions: List[str], dest: Optional[Path], cache: Cache
) -> int:
    # Once aioitertools has a new release, this can use concurency-limited
    # gather
    rc = 0
    coros = [async_download_one(package, v, dest, cache) for v in versions]
    for coro in asyncio.as_completed(coros):
        try:
            result = await coro
            print(result.as_posix())
        except Exception as e:
            click.secho(f"Error: {e}", fg="red")
            rc |= 1
    return rc


async def async_download_one(
    package: Package, version: str, dest: Optional[Path], cache: Cache
) -> Path:
    sdists = [
        f for f in package.releases[version].files if f.file_type == FileType.SDIST
    ]
    url = sdists[0].url
    cache_path = await cache.async_fetch(package.name, url)
    # TODO: check hash
    if dest:
        # So that cache can make arbitrary names, we get the basename portion
        # from the url.
        dest_filename = dest / posixpath.basename(url)
        # In the future, can reflink for additional speedup (pypi:reflink)
        # try:
        #     reflink.reflink(cache_path, dest_filename)
        # except (NotImplementedError, ReflinkImpossibleError) as e:
        shutil.copyfile(cache_path, dest_filename)
        return dest_filename
    return cache_path



import asyncio
import posixpath
import tempfile
import unittest
from pathlib import Path
from typing import Any, Dict, Optional, Tuple
from unittest import mock

from honesty.cache import Cache


class AiohttpStreamMock:
    def __init__(self, content: bytes) -> None:
        self._content = content

    # TODO async iterable[bytes]
    async def iter_any(self) -> Any:
        yield self._content


class AiohttpResponseMock:
    def __init__(self, content: bytes) -> None:
        self.content = AiohttpStreamMock(content)

    async def __aenter__(self) -> "AiohttpResponseMock":
        return self

    async def __aexit__(self, *args: Any) -> None:
        pass


class FakeCache:
    def __init__(
        self, path: str, url_to_contents: Dict[Tuple[str, Optional[str]], bytes]
    ) -> None:
        self.path: Path = Path(path)
        self.url_to_contents = url_to_contents

    async def async_fetch(self, package_name: str, url: Optional[str] = None) -> Path:
        basename = posixpath.basename(url) if url else f"{package_name}_index.html"
        with open(self.path / basename, "wb") as f:
            f.write(self.url_to_contents[(package_name, url)])

        return self.path / basename


class CacheTest(unittest.TestCase):
    def test_fetch_caches(self) -> None:

        d = tempfile.mkdtemp()

        def get_side_effect(
            url: str, raise_for_status: bool = False, timeout: Any = None
        ) -> AiohttpResponseMock:
            if url == "https://example.com/other":
                return AiohttpResponseMock(b"other")
            elif url == "https://pypi.org/a/relpath":
                return AiohttpResponseMock(b"relpath")
            elif url == "https://pypi.org/simple/projectname/":
                return AiohttpResponseMock(b"foo")

            raise NotImplementedError(url)  # pragma: no cover

        with Cache(index_url="https://pypi.org/simple/", cache_dir=d) as cache:

            with mock.patch.object(cache.session, "get", side_effect=get_side_effect):
                rv = cache.fetch("projectname", url=None)
                self.assertTrue(rv.exists(), rv)
                self.assertEqual(f"{d}/pr/oj/projectname/index.html", str(rv))
                rv = cache.fetch("projectname", url=None)
                self.assertEqual(f"{d}/pr/oj/projectname/index.html", str(rv))
                # TODO mock_get.assert_called_once()
                with rv.open() as f:
                    self.assertEqual("foo", f.read())

                # Absolute path url support
                rv = cache.fetch("projectname", url="https://example.com/other")
                with rv.open() as f:
                    self.assertEqual("other", f.read())

                # Relative path support
                rv = cache.fetch("projectname", url="../../a/relpath")
                with rv.open() as f:
                    self.assertEqual("relpath", f.read())

    def test_cache_defaults(self) -> None:
        with Cache() as cache:
            self.assertEqual(
                Path("~/.cache/honesty/pypi").expanduser(), cache.cache_path
            )
            self.assertEqual("https://pypi.org/simple/", cache.index_url)

    @mock.patch("honesty.cache.os.environ.get")
    def test_cache_env_vars(self, mock_get: Any) -> None:
        mock_get.side_effect = {
            "HONESTY_CACHE": "/tmp",
            "HONESTY_INDEX_URL": "https://example.com/foo",
        }.get
        with Cache() as cache:
            self.assertEqual(Path("/tmp"), cache.cache_path)
            self.assertEqual("https://example.com/foo/", cache.index_url)

    def test_cache_invalid(self) -> None:
        with Cache() as cache:
            with self.assertRaises(NotImplementedError):
                # I no longer remember which project triggers this; in theory
                # all non-[a-z0-9-] should have been canonicalized away already.
                cache.fetch("pb&amp;j", url=None)

    def test_aenter(self) -> None:
        async def inner() -> None:
            async with Cache() as cache:
                pass

        loop = asyncio.get_event_loop()
        loop.run_until_complete(inner())



import asyncio
import enum
import re
from dataclasses import dataclass
from typing import Any, Dict, List, Match, Optional, Tuple

from .cache import Cache

# Apologies in advance, "parsing" html via regex
ENTRY_RE = re.compile(
    r'href="(?P<url>[^"#]+\/(?P<basename>[^#]+))#(?P<checksum>[^="]+=[a-f0-9]+)"'
)
NUMERIC_VERSION = re.compile(
    r"^(?P<package>.*?)-(?P<version>[0-9][^-]*?)"
    r"(?P<suffix>(?P<platform>\.macosx|\.linux)?-.*)?$"
)


SDIST_EXTENSIONS = (".tar.gz", ".zip", ".tar.bz2")

# TODO IntFlag, or a separate field for platform
class FileType(enum.IntEnum):
    UNKNOWN = 0
    SDIST = 1
    BDIST_WHEEL = 2
    BDIST_EGG = 3
    BDIST_DUMB = 4


class UnexpectedFilename(Exception):
    pass


def guess_file_type(filename: str) -> FileType:
    if filename.endswith(".egg"):
        return FileType.BDIST_EGG
    elif filename.endswith(".whl"):
        return FileType.BDIST_WHEEL
    elif filename.endswith(SDIST_EXTENSIONS):
        filename = remove_suffix(filename)
        match = NUMERIC_VERSION.match(filename)
        # Some oddly-named files are not likely to be loaded by pip either.
        if match is None:
            raise UnexpectedFilename(filename)
        # bdist_dumb can't be easily discerned
        if match.group("platform"):
            return FileType.BDIST_DUMB
        elif match.group("suffix") and match.group("suffix").startswith("-macosx"):
            return FileType.BDIST_DUMB
        return FileType.SDIST
    else:  # .exe and .rpm at least
        return FileType.UNKNOWN


@dataclass
class FileEntry:
    url: str  # https://files.pythonhosted.../foo-1.0.tgz
    basename: str  # foo-1.0.tgz
    checksum: str  # 'sha256=<foo>'
    file_type: FileType
    version: str  # TODO: better type
    requires_python: Optional[str] = None  # '&gt;=3.6'
    # TODO extract upload date?

    @classmethod
    def from_entry_match(cls, match: Match[Any]) -> "FileEntry":
        d = match.groupdict()
        d["file_type"] = guess_file_type(d["basename"])
        d["version"] = guess_version(d["basename"])[1]
        obj = cls(**d)
        return obj


@dataclass
class PackageRelease:
    version: str
    files: List[FileEntry]


@dataclass
class Package:
    name: str
    releases: Dict[str, PackageRelease]


def remove_suffix(basename: str) -> str:
    suffixes = [".egg", ".whl", ".zip", ".gz", ".bz2", ".tar"]
    for s in suffixes:
        if basename.endswith(s):
            basename = basename[: -len(s)]
    return basename


# TODO itu-r-468-weighting-1.0.3.tar.gz
# TODO uttt-0.3-1.tar.gz
def guess_version(basename: str) -> Tuple[str, str]:
    """
    Returns (package name, version) or raises.
    """
    # This should use whatever setuptools/pip/etc use, but I spent about 10
    # minutes and couldn't find it tonight.
    basename = remove_suffix(basename)

    match = NUMERIC_VERSION.match(basename)
    if not match:
        raise UnexpectedFilename(basename)
    return match.group(1), match.group(2)


def parse_index(pkg: str, cache: Cache, strict: bool = False) -> Package:
    loop = asyncio.get_event_loop()
    package: Package = loop.run_until_complete(async_parse_index(pkg, cache, strict))
    return package


async def async_parse_index(pkg: str, cache: Cache, strict: bool = False) -> Package:
    package = Package(name=pkg, releases={})
    with open(await cache.async_fetch(pkg, url=None)) as f:
        for match in ENTRY_RE.finditer(f.read()):
            # TODO guess_version can also raise this, but they currently use the
            # same regex; this should get parsed out once maybe in a method on
            # FileEntry.
            try:
                fe = FileEntry.from_entry_match(match)
            except UnexpectedFilename:
                if not strict:
                    continue
                raise

            v = fe.version
            if v not in package.releases:
                package.releases[v] = PackageRelease(version=v, files=[])
            package.releases[v].files.append(fe)

    return package



import fnmatch
import hashlib
import os.path
import shutil
from pathlib import Path
from typing import Dict, Iterable, List, Tuple

ZIP_EXTENSIONS = (".zip", ".egg", ".whl")


def extract_and_get_names(
    archive_filename: Path,
    strip_top_level: bool = False,
    patterns: Iterable[str] = ("*.py",),
) -> Tuple[str, List[Tuple[str, str]]]:
    cache_path = os.path.expanduser(
        os.environ.get("HONESTY_EXTDIR", "~/.cache/honesty/ext")
    )
    archive_root = os.path.join(cache_path, archive_filename.name)
    if not os.path.exists(archive_root + ".done"):
        format = "zip" if str(archive_filename).endswith(ZIP_EXTENSIONS) else None
        # mypy-fixme: arg 1 expects str, not Path
        shutil.unpack_archive(archive_filename.as_posix(), archive_root, format)

    with open(archive_root + ".done", "w"):
        pass

    # relpath, srcpath
    names: List[Tuple[str, str]] = []
    # TODO figure out the right level of parallelism and/or use cfv
    for dirpath, dirnames, filenames in os.walk(archive_root):
        for name in filenames:
            if not any(fnmatch.fnmatch(name, p) for p in patterns):
                continue  #  skip for now

            relname = os.path.join(dirpath[len(archive_root) + 1 :], name)

            srckey = relname
            # To do this right, we need to read setup.py to know how it gets
            # mapped, but this is an 80% solution.
            if strip_top_level:
                srckey = srckey.split("/", 1)[-1]
            if srckey.startswith("src/"):
                srckey = srckey[4:]

            names.append((relname, srckey))

    return (archive_root, names)


# [path] = sha
def archive_hashes(
    archive_filename: Path, strip_top_level: bool = False
) -> Dict[str, str]:
    d: Dict[str, str] = {}
    archive_root, names = extract_and_get_names(archive_filename, strip_top_level)

    for relname, srcname in names:
        with open(os.path.join(archive_root, relname), "rb") as buf:
            data = buf.read().replace(b"\r\n", b"\n")

        sha = hashlib.sha1(data).hexdigest()
        d[srcname] = sha
    return d



import posixpath
import tempfile
import unittest
from pathlib import Path
from typing import Any
from unittest import mock

from honesty.releases import (
    FileType,
    UnexpectedFilename,
    guess_file_type,
    guess_version,
    parse_index,
)
from honesty.tests.cache import FakeCache

WOAH_INDEX_CONTENTS = b"""\
<!DOCTYPE html>
<html>
  <head>
    <title>Links for woah</title>
  </head>
  <body>
    <h1>Links for woah</h1>
    <a href="https://files.pythonhosted.org/packages/69/c9/a9951fcb2e706dd14cfc5d57a33eadc38a2b7477c82c12c229de5f6115db/woah-0.1-py3-none-any.whl#sha256=e705573ea8a88ec772174deea6a80c79f1e8b7e96130e27eee14b21d63f4e7f8" data-requires-python="&gt;=3.6">woah-0.1-py3-none-any.whl</a><br/>
    <a href="https://files.pythonhosted.org/packages/8f/3f/cd6d2edb9cf7049788db971fb5359cbde9fb28801d55b1aafa8f0df4813a/woah-0.1.tar.gz#sha256=d0760a3696271db53c361c950d93ceca7a022b5d739c0005e3bfb65785dd9d97" data-requires-python="&gt;=3.6">woah-0.1.tar.gz</a><br/>
    <a href="https://files.pythonhosted.org/packages/5e/95/871090fc9c10630d457b44967c9bb9c544b858cd3a2fe6dd60f9e169d99f/woah-0.2-py3-none-any.whl#sha256=e701a8d020a09fa32199cc74b386a3bf9730910fd46a6301fbb8203f287b27d7" data-requires-python="&gt;=3.6">woah-0.2-py3-none-any.whl</a><br/>
    <a href="https://files.pythonhosted.org/packages/fb/f2/dc6873f2763ffb457d3dbe4224ea59b21a8495fa0ef86d230b78cdba0f22/woah-0.2.tar.gz#sha256=62a886ed5e16506c039216dc0b5f342e72228e2038c750a1a7574321af6d8d68" data-requires-python="&gt;=3.6">woah-0.2.tar.gz</a><br/>
    </body>
</html>
<!--SERIAL 5860225-->
"""

LONG_NAME = "scipy-0.14.1rc1.dev_205726a-cp33-cp33m-macosx_10_6_intel.macosx_10_9_intel.macosx_10_9_x86_64.macosx_10_10_intel.macosx_10_10_x86_64.whl"


class ReleasesTest(unittest.TestCase):
    def test_get_entries(self) -> None:
        with tempfile.TemporaryDirectory() as d:
            c = FakeCache(d, {("woah", None): WOAH_INDEX_CONTENTS})
            pkg = parse_index("woah", c)  # type: ignore

        self.assertEqual("woah", pkg.name)
        self.assertEqual(2, len(pkg.releases))

        v01 = pkg.releases["0.1"]
        self.assertEqual(2, len(v01.files))

        self.assertEqual(
            "https://files.pythonhosted.org/packages/69/c9/a9951fcb2e706dd14cfc5d57a33eadc38a2b7477c82c12c229de5f6115db/woah-0.1-py3-none-any.whl",
            v01.files[0].url,
        )
        self.assertEqual("woah-0.1-py3-none-any.whl", v01.files[0].basename)
        self.assertEqual(
            "sha256=e705573ea8a88ec772174deea6a80c79f1e8b7e96130e27eee14b21d63f4e7f8",
            v01.files[0].checksum,
        )

    def test_strict(self) -> None:
        with tempfile.TemporaryDirectory() as d:
            c = FakeCache(
                d, {("woah", None): WOAH_INDEX_CONTENTS.replace(b"woah-0.1", b"woah")}
            )
            with self.assertRaises(UnexpectedFilename):
                pkg = parse_index("woah", c, strict=True)  # type: ignore

    def test_non_strict(self) -> None:
        with tempfile.TemporaryDirectory() as d:
            c = FakeCache(
                d, {("woah", None): WOAH_INDEX_CONTENTS.replace(b"woah-0.1", b"woah")}
            )
            pkg = parse_index("woah", c, strict=False)  # type: ignore

        self.assertEqual(1, len(pkg.releases))

        v02 = pkg.releases["0.2"]
        self.assertEqual(2, len(v02.files))

    def test_guess_version(self) -> None:
        self.assertEqual(("foo", "0.1"), guess_version("foo-0.1.tar.gz"))
        self.assertEqual(("foo", "0.1"), guess_version("foo-0.1-py3-none.whl"))
        self.assertEqual(("foo", "0.1"), guess_version("foo-0.1-any-none.whl"))
        with self.assertRaises(UnexpectedFilename):
            guess_version("foo.tar.gz")

        self.assertEqual(("scipy", "0.14.1rc1.dev_205726a"), guess_version(LONG_NAME))
        self.assertEqual(
            ("javatools", "1.4.0"),
            guess_version("javatools-1.4.0.macosx-10.14-x86_64.tar.gz"),
        )
        self.assertEqual(("pypi", "2"), guess_version("pypi-2.tar.gz"))

    def test_guess_file_type(self) -> None:
        self.assertEqual(FileType.SDIST, guess_file_type("foo-0.1.tar.gz"))
        self.assertEqual(
            FileType.BDIST_WHEEL, guess_file_type("foo-0.1-manylinux1.whl")
        )
        self.assertEqual(FileType.BDIST_EGG, guess_file_type("foo-0.1.egg"))
        self.assertEqual(
            FileType.BDIST_DUMB,
            guess_file_type("javatools-1.4.0.macosx-10.14-x86_64.tar.gz"),
        )
        self.assertEqual(FileType.UNKNOWN, guess_file_type("foo-0.1.exe"))
        self.assertEqual(
            FileType.BDIST_DUMB,
            guess_file_type("pyre-check-0.0.29-macosx_10_11_x86_64.tar.gz"),
        )
        self.assertEqual(FileType.SDIST, guess_file_type("pypi-2.tar.gz"))
        with self.assertRaises(UnexpectedFilename):
            guess_file_type("ibm_db.tar.gz")



import asyncio
import functools
import sys
from pathlib import Path
from typing import Any, List, Optional

import click
import pkg_resources

from honesty.__version__ import __version__
from honesty.api import async_download_many
from honesty.cache import Cache
from honesty.checker import has_nativemodules, is_pep517, run_checker
from honesty.releases import Package, async_parse_index, parse_index


# TODO type
def wrap_async(coro: Any) -> Any:
    @functools.wraps(coro)
    def inner(*args: Any, **kwargs: Any) -> Any:
        loop = asyncio.get_event_loop()
        return loop.run_until_complete(coro(*args, **kwargs))

    return inner


@click.group()
@click.version_option(__version__, prog_name="honesty")
def cli() -> None:
    pass


@cli.command(help="List available archives")
@click.option("--fresh", "-f", is_flag=True, type=bool)
@click.argument("package_name")
@wrap_async
async def list(fresh: bool, package_name: str) -> None:
    async with Cache(fresh_index=fresh) as cache:
        package = await async_parse_index(package_name, cache)

    print(f"package {package.name}")
    print("releases:")
    for k, v in package.releases.items():
        print(f"  {k}:")
        for f in v.files:
            print(f"    {f.basename}")


@cli.command(help="Check for consistency among archives")
@click.option("--verbose", "-v", is_flag=True, type=bool)
@click.option("--fresh", "-f", is_flag=True, type=bool)
@click.argument("package_name")
def check(verbose: bool, fresh: bool, package_name: str) -> None:
    with Cache(fresh_index=fresh) as cache:
        package_name, operator, version = package_name.partition("==")
        package = parse_index(package_name, cache)
        selected_versions = select_versions(package, operator, version)

        if verbose:
            click.echo(f"check {package_name} {selected_versions}")

        rc = 0
        for v in selected_versions:
            rc |= run_checker(package, v, verbose=verbose, cache=cache)

    if rc != 0:
        sys.exit(rc)


@cli.command(help="Check for presence of pep517 markers")
@click.option("--verbose", "-v", is_flag=True, type=bool)
@click.option("--fresh", "-f", is_flag=True, type=bool)
@click.argument("package_name")
def ispep517(verbose: bool, fresh: bool, package_name: str) -> None:
    with Cache(fresh_index=fresh) as cache:
        package_name, operator, version = package_name.partition("==")
        package = parse_index(package_name, cache)
        selected_versions = select_versions(package, operator, version)

        if verbose:
            click.echo(f"check {package_name} {selected_versions}")

        rc = 0
        for v in selected_versions:
            rc |= is_pep517(package, v, verbose=verbose, cache=cache)

    if rc != 0:
        sys.exit(rc)


@cli.command(help="Check for native modules in bdist")
@click.option("--verbose", "-v", is_flag=True, type=bool)
@click.option("--fresh", "-f", is_flag=True, type=bool)
@click.argument("package_name")
def native(verbose: bool, fresh: bool, package_name: str) -> None:
    with Cache(fresh_index=fresh) as cache:
        package_name, operator, version = package_name.partition("==")
        package = parse_index(package_name, cache)
        selected_versions = select_versions(package, operator, version)

        if verbose:
            click.echo(f"check {package_name} {selected_versions}")

        rc = 0
        for v in selected_versions:
            rc |= has_nativemodules(package, v, verbose=verbose, cache=cache)

    if rc != 0:
        sys.exit(rc)


@cli.command(help="Download an sdist, print path on stdout")
@click.option("--verbose", "-v", is_flag=True, type=bool)
@click.option("--fresh", "-f", is_flag=True, type=bool)
@click.option("--dest", help="Directory to store in", default="")
@click.option(
    "--index-url", help="Alternate index url (uses HONESTY_INDEX_URL or pypi by default"
)
@click.argument("package_name")
@wrap_async
async def download(
    verbose: bool, fresh: bool, dest: str, index_url: Optional[str], package_name: str
) -> None:
    dest_path: Optional[Path]
    if dest:
        dest_path = Path(dest)
        dest_path.mkdir(parents=True, exist_ok=True)
    else:
        dest_path = None

    async with Cache(fresh_index=fresh, index_url=index_url) as cache:
        package_name, operator, version = package_name.partition("==")
        package = await async_parse_index(package_name, cache)
        selected_versions = select_versions(package, operator, version)

        if verbose:
            click.echo(f"check {package_name} {selected_versions}")

        rc = await async_download_many(
            package, versions=selected_versions, dest=dest_path, cache=cache
        )

    sys.exit(rc)


def select_versions(package: Package, operator: str, selector: str) -> List[str]:
    """
    Given operator='==' and selector='*' or '2.0', return a list of the matching
    versions, in increasing order.
    """
    if not package.releases:
        raise click.ClickException(f"No releases at all for {package.name}")

    if operator not in ("", "=="):
        raise click.ClickException("Only '==' is supported")

    if selector == "":
        # latest
        version = sorted(package.releases, key=pkg_resources.parse_version)[-1]
        return [version]
    elif selector == "*":
        versions: List[str] = sorted(package.releases, key=pkg_resources.parse_version)
        return versions
    else:
        if selector not in package.releases:
            raise click.ClickException(
                f"The version {selector} does not exist for {package.name}"
            )
        return [selector]


if __name__ == "__main__":
    cli()



import difflib
import hashlib
import os.path
import time
from pathlib import Path
from typing import Dict, List, Set, Tuple

import click

from .archive import archive_hashes, extract_and_get_names
from .cache import Cache
from .releases import SDIST_EXTENSIONS, FileEntry, FileType, Package


def run_checker(package: Package, version: str, verbose: bool, cache: Cache) -> int:
    try:
        rel = package.releases[version]
    except KeyError:
        raise click.ClickException(f"version={version} not available")

    sdists = [f for f in rel.files if f.file_type == FileType.SDIST]

    if not sdists:
        click.secho(f"{package.name} {version} no sdist", fg="red")
        return 1
    elif len(sdists) == len(rel.files):
        click.secho(f"{package.name} {version} only sdist", fg="green")
        return 0

    local_paths: List[Tuple[FileEntry, Path]] = []
    with click.progressbar(rel.files) as bar:
        for fe in bar:
            local_paths.append((fe, cache.fetch(pkg=package.name, url=fe.url)))
            # TODO verify checksum

    sdist_hashes: Dict[str, str] = {}
    for fe, lp in local_paths:
        if fe.file_type == FileType.SDIST:
            # assert not sdist_hashes # multiple sdists?
            t0 = time.time()
            sdist_hashes = archive_hashes(lp, True)
            t1 = time.time()
            if verbose:
                print(f"{fe.basename} {t1-t0}")

    if verbose:
        for k, v in sdist_hashes.items():
            print(f"{k} {v}")

    # [message] = set(filenames)
    messages: Dict[str, Set[str]] = {}
    rc = 0
    for fe, lp in local_paths:
        if fe.file_type in (FileType.BDIST_WHEEL, FileType.BDIST_EGG):
            t0 = time.time()
            this_hashes = archive_hashes(lp)
            t1 = time.time()
            if verbose:
                print(f"{fe.basename} {t1-t0}")

            msg = []
            for k, h in sorted(this_hashes.items()):
                if k not in sdist_hashes:
                    # Intentionally not including has here, because
                    # scipy/__config__.py has a different hash in each one and
                    # I want them to coalesce
                    msg.append(f"    {k} not in sdist")
                    rc |= 4
                elif h != sdist_hashes[k]:
                    msg.append(f"    {k} differs from sdist {h}")
                    rc |= 8

            if msg:
                messages.setdefault("\n".join(msg), set()).add(fe.basename)

    if rc == 0:
        click.secho(f"{package.name} {version} OK", fg="green")
    else:
        click.secho(f"{package.name} {version} problems", fg="yellow")
        for k, vm in messages.items():
            for i in vm:
                click.secho(f"  {i}", fg="red")
            click.secho(k, fg="yellow")

    return rc


def is_pep517(package: Package, version: str, verbose: bool, cache: Cache) -> bool:
    try:
        rel = package.releases[version]
    except KeyError:
        raise click.ClickException(f"version={version} not available")

    # Find *a* sdist
    sdists = [f for f in rel.files if f.file_type == FileType.SDIST]
    if not sdists:
        raise click.ClickException(f"{package.name} no sdists")

    lp = cache.fetch(pkg=package.name, url=sdists[0].url)

    archive_root, names = extract_and_get_names(
        lp, strip_top_level=True, patterns=("pyproject.toml",)
    )
    for relname, srcname in names:
        # TODO for a couple of projects this is finding test fixtures, we
        # should only be looking alongside the rootmost setup.py
        if srcname.endswith("pyproject.toml"):
            with open(os.path.join(archive_root, relname), "rb") as buf:
                data = buf.read().replace(b"\r\n", b"\n")
            if b"[build-system]" in data:
                click.echo(f"{package.name} build-system {relname}")
                return True
            else:
                click.echo(f"{package.name} has-toml {relname}")
    return False


def has_nativemodules(
    package: Package, version: str, verbose: bool, cache: Cache
) -> bool:
    try:
        rel = package.releases[version]
    except KeyError:
        raise click.ClickException(f"version={version} not available")

    # Find *a* sdist
    bdists = [f for f in rel.files if f.file_type == FileType.BDIST_WHEEL]
    if not bdists:
        raise click.ClickException(f"{package.name} no bdists")

    if verbose:
        click.echo(f"{package.name} {version} {bdists[0].basename}")

    lp = cache.fetch(pkg=package.name, url=bdists[0].url)

    archive_root, names = extract_and_get_names(
        lp, strip_top_level=False, patterns=("*.so", "*.dll")
    )
    for relname, srcname in names:
        # TODO for a couple of projects this is finding test fixtures, we
        # should only be looking alongside the rootmost setup.py
        if srcname.endswith(".so") or srcname.endswith(".dll"):
            click.echo(f"{package.name} has {srcname}")
            return True

    return False


def shorten(subj: str, n: int = 50) -> str:
    if len(subj) <= n:
        return subj
    return subj[:22] + "..." + subj[-n + 22 + 3 :]


def show_diff(a: List[str], b: List[str]) -> None:
    click.echo("".join(difflib.unified_diff(a, b)))



from setuptools import find_packages, setup

with open("README.md") as f:
    readme = f.read()

with open("honesty/__version__.py") as f:
    version = f.read().split()[-1].strip('"')

setup(
    name="honesty",
    version=version,
    description="Double check sdist/bdist on pypi",
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Tim Hatch",
    author_email="tim@timhatch.com",
    url="https://github.com/thatch/honesty",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Topic :: Utilities",
    ],
    license="Apache 2.0",
    packages=find_packages(),
    python_requires=">=3.6",
    entry_points={"console_scripts": ["honesty = honesty.cmdline:cli"]},
    install_requires=[
        "aiohttp >= 3.6",
        "click >= 7.0",
        "dataclasses >= 0.7; python_version < '3.7'",
    ],
)
